<?php

namespace Drupal\vais_promos\Form;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for the vais_promo entity edit forms.
 *
 * @ingroup vais_promos
 */
class VaisPromoForm extends ContentEntityForm {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The constructor for the Form.
   *
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   */
  public function __construct(EntityRepositoryInterface $entity_repository, EntityTypeBundleInfoInterface $entity_type_bundle_info, TimeInterface $time, Connection $database) {
    parent::__construct(
      $entity_repository,
      $entity_type_bundle_info,
      $time
    );
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {

    return new static(
      $container->get('entity.repository'),
      $container->get('entity_type.bundle.info'),
      $container->get('datetime.time'),
      $container->get('database')
    );

  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\vais_promos\Entity\VaisPromo $entity */
    $form = parent::buildForm($form, $form_state);
    $entity = $this->entity;

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    $entity = parent::validateForm($form, $form_state);

    // Get the search page to which this promotion belongs.
    $promoSearchPage = $form_state->getValue('promo_search_page')[0]['value'];

    // Get the promotion trigger words.
    $triggers = explode(",", str_replace(" ", "", $form_state->getValue('promo_trigger')[0]['value']));

    // Get all promotions associate to same search page.
    $query = $this->database->select('vais_promo', 'vp');
    $query->condition('vp.promo_search_page', $promoSearchPage, '=');

    // Do not get the promo currently being updated.
    if (!$entity->isNew()) {
      $query->condition('pid', $entity->id(), '!=');
    }

    $query->addField('vp', 'pid', 'pid');
    $query->addField('vp', 'promo_trigger', 'trigger');
    $results = $query->execute()->fetchAll();

    // Determine if other promotion is using trigger words already.
    foreach ($results as $promo) {

      $promoTriggers = explode(",", str_replace(" ", "", $promo->trigger));
      $duplicate = array_values(array_intersect($triggers, $promoTriggers));

      // If trigger word is not unique, set error on form.
      if (!empty($duplicate)) {
        $triggerLink = Link::createFromRoute(
          'another promotion',
          'entity.vais_promo.edit_form',
          ['vais_promo' => $promo->pid]
        );
        $form_state->setErrorByName('promo_trigger',
          $this->t(
            'The trigger word, ":triggerWord" is used in @triggerLink.',
            [
              ':triggerWord' => $duplicate[0],
              '@triggerLink' => $triggerLink->toString(),
            ]
          )
        );
        break;
      }

    }

    // Ensure numeric values.
    if ($form_state->hasValue('weight') && !is_numeric($form_state->getValue('weight'))) {
      $form_state->setErrorByName('weight', $this->t('Weight value must be numeric.'));
    }

    return $entity;

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $promoTrigger = $form_state->getValue('promo_trigger')[0]['value'];

    // Set all trigger words to lowercase.
    $form_state->setValue(
      'promo_trigger',
        [['value' => strtolower($promoTrigger)]]
    );

    parent::submitForm($form, $form_state);

  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $status = parent::save($form, $form_state);

    $entity = $this->entity;
    if ($status == SAVED_UPDATED) {
      $this->messenger()
        ->addMessage($this->t('The promo %feed has been updated.', ['%feed' => $entity->toLink()->toString()]));
    }
    else {
      $this->messenger()
        ->addMessage($this->t('The promo %feed has been added.', ['%feed' => $entity->toLink()->toString()]));
    }

    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $status;
  }

}
